//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package scalablewavelets;

import ij.IJ;
import ij.ImagePlus;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import scalablewavelets.fft.AcademicFFT;
import spotcaliper.detector.Spot;
import spotcaliper.gui.components.ProgressionBar;
import spotcaliper.image2d.Image2DDouble;

public class Analysis {
	private final static double		EPSILONPRIME	= 0.3;
	private final static double		LEFTEND			= 1.0 / Math.pow(4.0, (1.0 + EPSILONPRIME));
	private final static double		RIGHTEND		= 1.0 / Math.pow(4., (0.5 + EPSILONPRIME));
	private final static double		SIGMA			= 2.0;
	private final static double		COEF			= Math.sqrt(9370.0) / 14055.0;
	private final static double[]	ALPHA			= new double[] { COEF * (125.0 / Math.sqrt(2)), COEF * 101.0, COEF * 53.0, COEF * 16.0, COEF * 2.0 };

	private int						nx;
	private int						ny;
	private int						nchannels;
	private int						nscales_;
	private ProgressionBar			progress;
	public Image2DDouble[][]		analyzedImages;
	public Image2DDouble			maximumMagnitude;

	private double					delta			= 100;
	private boolean					multithread		= true;
	private boolean					showAnalysis	= false;
	private boolean					stop			= false;

	public static int getNumberOfScales(double radMax) {
		return (int) Math.max(1, Math.ceil(-MathScalable.log2((Math.pow(4, 1 + EPSILONPRIME) / (2 * radMax)))));
	}

	public static int getNumberOfChannel() {
		return 2 * (ALPHA.length - 1) + 1;
	}

	public Analysis(ProgressionBar progress, int nx, int ny, double maxRad, double delta, boolean showAnalysis, boolean multithread) {
		this.progress = progress;
		this.showAnalysis = showAnalysis;
		this.multithread = multithread;
		this.delta = delta;
		this.nx = nx;
		this.ny = ny;
		this.nchannels = getNumberOfChannel();
		this.nscales_ = getNumberOfScales(maxRad);
	}

	public void stop() {
		stop = true;
	}

	public void analyze(double input[]) {

		double[] reData = new double[input.length];
		double[] imData = new double[input.length];
		for (int k = 0; k < input.length; ++k)
			reData[k] = input[k];

		AcademicFFT fft = new AcademicFFT(nx, ny, 0, 0);
		fft.directTransform(reData, imData, null, null, AcademicFFT.InputDataType.REALINPUT);

		if (stop)
			return;
		Image2DDouble[] primalReal = new Image2DDouble[nscales_];
		Image2DDouble[] primalImag = new Image2DDouble[nscales_];
		double R[] = MathScalable.buildRF(nx, ny);
	
		WaveletFilters filters_ = new WaveletFilters(nx, ny);
		for (int sc = 0; sc < nscales_; ++sc) {
			primalReal[sc] = filters_.ward(R, false, Math.pow(2, -sc - 1));
			primalReal[sc].multiply(filters_.ward(R, true, Math.pow(2, -sc - 3)));
			primalImag[sc] = new Image2DDouble(nx, ny);
			multiplyImaginaryZero(primalReal[sc], primalImag[sc], reData, imData);
			if (stop)
				return;
		}

		Filter filter[] = new Filter[nchannels];
		for (int channel = 0; channel < nchannels; ++channel) {
			progress.progress("Multipliers " + (channel + 1) + "/" + nchannels, (channel + 1) * 100.0 / nchannels);
			double val = SIGMA * channel / nchannels;
			filter[channel] = new Filter(channel, progress, nscales_, R, primalReal, primalImag, val, ALPHA);
		}

		if (multithread) {
			int cores = Runtime.getRuntime().availableProcessors();
			ExecutorService executor = Executors.newFixedThreadPool(cores);
			for (int channel = 0; channel < nchannels; ++channel)
				executor.execute(filter[channel]);
			executor.shutdown();
			while (!executor.isTerminated())
				;
		}
		else {
			for (int channel = 0; channel < nchannels; ++channel)
				filter[channel].run();
		}

		analyzedImages = new Image2DDouble[nscales_][nchannels];
		for (int channel = 0; channel < nchannels; ++channel) {
			Image2DDouble results[] = filter[channel].getResults();
			for (int sc = 0; sc < nscales_; ++sc)
				analyzedImages[sc][channel] = results[sc];
			if (stop)
				return;
		}

		if (showAnalysis) {
			ImagePlus out = IJ.createHyperStack("Analyzed Images", nx, ny, nchannels, nscales_, 1, 32);
			for (int channel = 0; channel < nchannels; ++channel)
				for (int scale = 0; scale < nscales_; ++scale) {
					out.setPosition(channel, scale, 0);
					out.setProcessor(analyzedImages[scale][channel].buildImageStack().getProcessor(1));
				}
			out.show();
		}
	}
	
	public double[][] computeMap() {

		maximumMagnitude = new Image2DDouble(nx, ny);
		double pixmax[] = maximumMagnitude.getData();
		double[][] map = new double[nx][ny];

		double pixmagnitude[] = new double[nx*ny];
		for (int sc = 0; sc < nscales_; ++sc) {
			progress.progress("Estimation Scale " + sc, sc * 100. / nscales_);
			for (int i = 0; i < analyzedImages[sc].length; ++i) {
				double[] ai = analyzedImages[sc][i].getData();
				for (int k = 0; k < pixmax.length; ++k) {
					pixmagnitude[k] += ai[k] * ai[k];
				}
			}
			if (stop)
				return map;
			for (int k = 0; k < pixmax.length; ++k) {
				pixmagnitude[k] = Math.sqrt(pixmagnitude[k]);
				if (pixmagnitude[k] > pixmax[k])
					pixmax[k] = pixmagnitude[k];
			}
		}

		if (stop)
			return map;

		for (int i = 0; i < nx; ++i)
			for (int j = 0; j < ny; ++j)
				map[i][j] = pixmax[i+j*nx];
		progress.progress("Wavelet map ", 99);
		return map;

	}

	/*
	public double[][] computeMap(double percent) {

		Image2DDouble[] magnitude = new Image2DDouble[nscales_];
		double[] maxval = new double[nscales_];
		maximumMagnitude = new Image2DDouble(nx, ny);
		double pixm[] = maximumMagnitude.getData();
		double[][] map = new double[nx][ny];

		for (int sc = 0; sc < nscales_; ++sc) {
			progress.progress("Estimation " + sc, sc * 100. / nscales_);
			magnitude[sc] = new Image2DDouble(nx, ny);
			double pixmagnitude[] = magnitude[sc].getData();
			for (int i = 0; i < analyzedImages[sc].length; ++i) {
				double[] ai = analyzedImages[sc][i].getData();
				for (int k = 0; k < pixm.length; ++k) {
					pixmagnitude[k] += ai[k] * ai[k];
				}
			}
			if (stop)
				return map;
			magnitude[sc].sqrt();
			double max = -Double.MAX_VALUE;
			for (int k = 0; k < pixm.length; ++k) {
				if (pixmagnitude[k] > pixm[k])
					pixm[k] = pixmagnitude[k];
				if (pixmagnitude[k] > max)
					max = pixmagnitude[k];
			}
			maxval[sc] = max;
		}

		double T[] = new double[nscales_];
		if (stop)
			return map;

		for (int sc = 0; sc < nscales_; ++sc)
			T[sc] = percent * maxval[sc];
		for (int ii = 0; ii < nx; ++ii)
			for (int jj = 0; jj < ny; ++jj)
				for (int sc = 0; sc < nscales_; ++sc)
					if (magnitude[sc].getPixel(ii, jj) > T[sc]) {
						map[ii][jj] = maximumMagnitude.getPixel(ii, jj);
						break;
					}
		return map;

	}
	*/
	public Spot getOptimalScale(int id, double[] coord) {
		int x = (int)Math.round(coord[0]);
		int y = (int)Math.round(coord[1]);
		double rho[] = new double[nchannels];
		for (int channel = 0; channel < nchannels; ++channel) {
			double val = SIGMA * channel / nchannels;
			rho[channel] = Math.pow(2, val);
		}
		double min_loc = 100;
		double min_value = Double.MAX_VALUE;
		boolean flag1 = false;
		boolean flag2 = false;
		double maxmagnitude = maximumMagnitude.getPixel(x, y);
		for (int sc = nscales_ - 1; sc >= 0; --sc) {
			double leftendSc = Math.pow(2, -sc) * LEFTEND;
			double rightendSc = Math.pow(2, -sc) * RIGHTEND;
			double step = (rightendSc - leftendSc) / delta;
			double[] anImVec = extractVec(analyzedImages, sc, x, y);
			for (double d = leftendSc + step; d <= rightendSc; d += step) {	
				double funval = evaluateFunction(anImVec, d, rho, ALPHA);
				if (funval > 0)
					flag1 = true;
				if (funval < 0 && flag1)
					flag2 = true;
				if (flag2 && (funval > 0.75 * min_value))
					return new Spot(id, x, y, 1.12 * 0.75 / min_loc, maxmagnitude, 0, 0);
				if (funval < min_value) {
					min_value = funval;
					min_loc = d;
				}
			}
		}
		return null;
	}

	private double evaluateFunction(double[] anImVec, double d, double rho_[], double alpha_[]) {
		double funval = 0.0;
		double kv[] = MathScalable.kernelvector(d, rho_, alpha_);
		for (int chan = 0; chan < anImVec.length; ++chan) {
			funval += kv[chan] * anImVec[chan];
		}
		return funval;
	}

	private double[] extractVec(Image2DDouble[][] iw, int sc, int ii, int jj) {
		double[] vec = new double[iw[0].length];
		for (int v = 0; v < iw[0].length; ++v) {
			vec[v] = iw[sc][v].getPixel(ii, jj);
		}
		return vec;
	}

	public void multiply(double areal[], double aimag[], double[] breal, double bimag[]) {
		int n = breal.length;
		for (int k = 0; k < n; k++) {
			double r = breal[k];
			double p = bimag[k];
			double imagvalue = r * aimag[k] + p * areal[k];
			double realvalue = r * areal[k] - p * aimag[k];
			aimag[k] = imagvalue;
			areal[k] = realvalue;
		}
	}

	public void multiplyImaginaryZero(Image2DDouble ireal, Image2DDouble iimag, double[] breal, double bimag[]) {
		int n = breal.length;
		double areal[] = ireal.getData();
		double aimag[] = iimag.getData();
		for (int k = 0; k < n; k++) {
			aimag[k] = bimag[k] * areal[k];
			areal[k] = breal[k] * areal[k];
		}
	}
	/*
	 * private void locMax(double percent) { Image2DDouble[] magnitude = new
	 * Image2DDouble[nscales_]; double[] maxval = new double[nscales_]; m = new
	 * Image2DDouble(nx, ny); double pixm[] = m.getData(); int[] bestScale = new
	 * int[nx*ny];
	 * 
	 * for (int sc = 0; sc < nscales_; ++sc) { progress.progress("Estimation " +
	 * sc, sc * 100. / nscales_); magnitude[sc] = new Image2DDouble(nx, ny);
	 * double pixmagnitude[] = magnitude[sc].getData(); for (int i = 0; i <
	 * analyzedImages[sc].length; ++i) { double[] ai =
	 * analyzedImages[sc][i].getData(); for (int k = 0; k < pixm.length; ++k) {
	 * pixmagnitude[k] += ai[k] * ai[k]; } } magnitude[sc].sqrt(); double max =
	 * -Double.MAX_VALUE; for (int k = 0; k < pixm.length; ++k) { if
	 * (pixmagnitude[k] > pixm[k]) { pixm[k] = pixmagnitude[k]; bestScale[k] =
	 * sc; } if (pixmagnitude[k] > max) max = pixmagnitude[k]; } maxval[sc] =
	 * max; }
	 * 
	 * double T[] = new double[nscales_];
	 * 
	 * for (int sc = 0; sc < nscales_; ++sc) T[sc] = percent * maxval[sc];
	 * double[][] LocMax = new double[nx][ny]; for (int ii = 0; ii < nx; ++ii)
	 * for (int jj = 0; jj < ny; ++jj) { for (int sc = 0; sc < nscales_; ++sc) {
	 * if (magnitude[sc].getPixel(ii, jj) > T[sc]) { LocMax[ii][jj] =
	 * m.getPixel(ii, jj); break; } } } ArrayList<int[]> candidatePixels = new
	 * ArrayList<int[]>(); candidatePixels = locmax3x3(LocMax);
	 * 
	 * candidateList_ = new ArrayList<Spot>(); double EPSILONPRIME = 0.3; double
	 * LFTEND_ = 1.0 / Math.pow(4, (1 + EPSILONPRIME)); double RIGHTEND = 1.0 /
	 * Math.pow(4, (0.5 + EPSILONPRIME));
	 * 
	 * for (int c = 0; c < candidatePixels.size(); c++) { int i =
	 * candidatePixels.get(c)[0]; int j = candidatePixels.get(c)[1]; int sc =
	 * bestScale[i + j*nx]; double leftendSc = Math.pow(2, -sc) * LFTEND_;
	 * double rightendSc = Math.pow(2, -sc) * E; double radius = 1.12 * 0.75 /
	 * (leftendSc + (rightendSc-leftendSc)*0.5); candidateList_.add(new Spot(c +
	 * 1, i, j, radius, LocMax[i][j], 10, 10)); } }
	 */

}
