//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package scalablewavelets;

import scalablewavelets.fft.AcademicFFT;
import spotcaliper.gui.components.ProgressionBar;
import spotcaliper.image2d.Image2DDouble;

public class Filter implements Runnable {

	private int					nscales;
	private int					nx;
	private int					ny;
	private double				R[];
	private Image2DDouble[]		primalReal;
	private Image2DDouble[]		primalImag;
	private double				val;
	private double				alpha[];
	private Image2DDouble		wd_real[];
	private ProgressionBar	progress;
	private int channel;
	
	public Filter(int channel, final ProgressionBar progress, final int nscales, double R[], final Image2DDouble[] primalReal, final Image2DDouble[] primalImag, final double val,
			final double alpha[]) {
		this.channel = channel;
		this.nscales = nscales;
		this.progress = progress;
		this.nx = primalReal[0].getWidth();
		this.ny = primalReal[0].getHeight();
		this.R = R;
		this.primalReal = primalReal;
		this.primalImag = primalImag;
		this.val = val;
		this.alpha = alpha;
		this.wd_real = new Image2DDouble[nscales];

	}

	public Image2DDouble[] getResults() {
		return wd_real;
	}

	@Override
	public void run() {
		double rho = Math.pow(2, val);
		double multiplier[] = Multipliers.channelIn1DArray(R, alpha, rho, nx, ny);
		AcademicFFT fft = new AcademicFFT(nx, ny, 0, 0);
		for (int sc = 0; sc < nscales; ++sc) {
			double reFFTData[] = multiply(primalReal[sc], multiplier);
			double imFFTData[] = multiply(primalImag[sc], multiplier);
			progress.progress("Channel " + channel + " Scale "+ sc, sc*100.0/nscales);
			reFFTData[0] = 0; // set 0 in (0,0)
			imFFTData[0] = 0; // set 0 in (0,0)
			fft.inverseTransform(reFFTData, imFFTData, null, null);
			wd_real[sc] = new Image2DDouble(reFFTData, nx, ny);
		}
	}

	private double[] multiply(Image2DDouble a, double[] b) {
		double ad[] = a.getData();
		int n = ad.length;
		double cd[] = new double[n];
		for (int k = 0; k < n; k++)
			cd[k] = ad[k] * b[k];
		return cd;
	}

}
