//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package scalablewavelets;

public class MathScalable {

	public static double[] linspace(double min, double max, int num) {
		double[] vec = new double[num];
		for (int i = 0; i < num; ++i)
			vec[i] = i * ((max - min) / num) + min;
		return vec;
	}

	public static double[] buildRF(int nx, int ny) {
		double[] x = linspace(0, 1, nx);
		double[] y = linspace(0, 1, ny);
		double[] dist = new double[nx * ny];
		for (int i = 0; i < nx; ++i)
			for (int j = 0; j < ny; ++j) {
				double val1 = x[i] - Math.floor(2.0 * x[i]);
				double val2 = y[j] - Math.floor(2.0 * y[j]);
				dist[i + nx * j] = Math.sqrt(val1 * val1 + val2 * val2);
			}
		return dist;
	}

	public static double[] kernelvector(double newpt, double[] rho, double[] alpha) {
		int lmax = alpha.length - 1;
		int nmax = 2 * lmax + 1;
		double[] y = new double[nmax];
		double xval;
		for (int ii = 0; ii < nmax; ++ii) {
			xval = rho[ii] / newpt;
			y[ii] = kernelprofile(xval, alpha);
		}
		return y;
	}

	public static double log2(double val) {
		return Math.log(val) / Math.log(2);
	}

	private static double kernelprofile(double xval, double[] alpha) {
		int lmax = alpha.length - 1;
		int nmax = 2 * lmax + 1;
		double y = 1;
		for (int l = 1; l <= lmax; ++l) {
			y += 2 * Math.cos(Math.PI * l * log2(xval));
		}
		y = y / nmax;
		return y;
	}

}
