//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package scalablewavelets;

import spotcaliper.image2d.Image2DDouble;

public class WaveletFilters {

	private double	EPS_	= 0.1;		// transmission region size
	private double	LE_		= 1 - EPS_; // left endpoint of transition
	private double	RE_		= 1;		// right endpoint of transition

	private double	q2_le	= q2(LE_);

	private int		nx_;
	private int		ny_;

	public WaveletFilters(int nx, int ny) {
		this.ny_ = ny;
		this.nx_ = nx;
	}

	public Image2DDouble ward(double R[], boolean lh, double cutoff) {
		Image2DDouble y = new Image2DDouble(nx_, ny_);
		for (int i = 0; i < nx_; ++i) {
			for (int j = 0; j < ny_; ++j) {
				y.putPixel(i, j, wardprofile(R[i + nx_ * j], lh, cutoff));
			}
		}
		return y;
	}

	public Image2DDouble ward(double R[][], boolean lh, double cutoff) {
		Image2DDouble y = new Image2DDouble(nx_, ny_);
		for (int i = 0; i < nx_; ++i) {
			for (int j = 0; j < ny_; ++j) {
				y.putPixel(i, j, wardprofile(R[i][j], lh, cutoff));
			}
		}
		return y;
	}

	public double wardprofile(double xval, boolean lh, double cutoff) {

		// rescale for cutoff value
		double x = xval / cutoff;
		double y = 0;
		// low-pass (lh=0) and high-pass (lh=1) filters
		if (!lh) {
			if (x <= LE_) {
				y = 1.0;
			}
			if ((x > LE_) & (x <= RE_)) {
				y = Math.sin((Math.PI / 2.0) * q3(x));
			}
			if (x > RE_) {
				y = 0.0;
			}

		}
		else {
			if (x <= LE_) {
				y = 0.0;
			}
			if ((x > LE_) & (x <= RE_)) {
				y = Math.cos((Math.PI / 2.0) * q3(x));
			}
			if (x > RE_) {
				y = 1.0;
			}
		}
		return y;

	}

	private double q3(double x) {
		double q3 = q2(x) / q2_le;
		return q3;
	}

	private double q2(double x) {
		double q2 = 0.2 * Math.pow(x, 5) - 0.95 * Math.pow(x, 4) + (541.0 / 300.0) * Math.pow(x, 3) - 1.71 * Math.pow(x, 2) + 0.81 * x - 23.0 / 150.0;
		return q2;
	}

}
