//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package spotcaliper.detector;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import spotcaliper.image2d.Image2DDouble;

public class Spot {

	public int		id;
	public double	x;
	public double	y;
	public double	r;
	public double	a;
	public double	contrast;
	public double	snr;
	private String	type	= "Auto";

	public Spot(int id, double x, double y, double r, double a, double contrast, double snr, String type) {
		this.id = id;
		this.x = x;
		this.y = y;
		this.r = r;
		this.a = a;
		this.contrast = contrast;
		this.snr = snr;
		this.type = type;
	}

	public Spot(int id, double x, double y, double r, double a, double contrast, double snr) {
		this.id = id;
		this.x = x;
		this.y = y;
		this.r = r;
		this.a = a;
		this.contrast = contrast;
		this.snr = snr;
	}

	public boolean isManual() {
		return !type.equals("Auto");
	}

	public String getType() {
		return type;
	}

	public double[] getFeature() {
		return new double[] { x, y, r, a, contrast, snr };
	}

	public void setManual() {
		DateFormat df = new SimpleDateFormat("h:m:s");
		type = "Manual " + df.format(new Date());
	}

	public double distance(double xp, double yp) {
		return Math.sqrt((x - xp) * (x - xp) + (y - yp) * (y - yp));
	}

	public boolean contains(int xm, int ym) {
		return (distance(xm, ym) <= r);
	}

	public double overlap(Spot spot) {
		double d = Math.sqrt((x - spot.x) * (x - spot.x) + (y - spot.y) * (y - spot.y));
		return d - r - spot.r;
	}

	public void measure(Image2DDouble iw) {
		double signal = measureAverageSignal(iw, x, y, r);
		double stats[] = measureStatsBackground(iw, x, y, r, Math.sqrt(2) * r);
		contrast = Math.abs(signal - stats[0]);
		snr = 20.0 * Math.log10(contrast*contrast / (stats[1] + 10e-8));
	}

	private double[] measureStatsBackground(Image2DDouble iw, double x, double y, double rin, double rout) {
		double avg = 0.0;
		double sqr = 0.0;
		int n = 0;
		int nx = iw.getWidth();
		int ny = iw.getHeight();
		int min_ii = (int) ((x - rout > 0) ? (x - rout) : 0);
		int max_ii = (int) ((x + rout < nx) ? (x + rout) : nx);
		int min_jj = (int) ((y - rout > 0) ? (y - rout) : 0);
		int max_jj = (int) ((y + rout < ny) ? (y + rout) : ny);
		double rout2 = rout * rout;
		double rin2 = rin * rin;
		for (int ii = min_ii; ii <= max_ii; ++ii)
			for (int jj = min_jj; jj <= max_jj; ++jj) {
				double d2 = (x - ii) * (x - ii) + (y - jj) * (y - jj);
				if ( d2 <= rout2) 
				if ( d2 > rin2) {
					n++;
					double v = iw.getPixel(ii, jj);
					avg += v;
					sqr += v * v;
				}
			}
		return new double[] { avg / n, Math.abs(n * sqr - avg * avg) / (n * n) };
	}

	private double measureAverageSignal(Image2DDouble iw, double x, double y, double R) {
		double avg = 0;
		int num_el = 0;
		int nx = iw.getWidth();
		int ny = iw.getHeight();
		int min_ii = (int) ((x - R > 0) ? (x - R) : 0);
		int max_ii = (int) ((x + R < nx) ? (x + R) : nx);
		int min_jj = (int) ((y - R > 0) ? (y - R) : 0);
		int max_jj = (int) ((y + R < ny) ? (y + R) : ny);
		for (int ii = min_ii; ii <= max_ii; ++ii)
			for (int jj = min_jj; jj <= max_jj; ++jj) {
				if ((x - ii) * (x - ii) + (y - jj) * (y - jj) <= R * R) {
					++num_el;
					avg += iw.getPixel(ii, jj);
				}
			}
		return avg / num_el;
	}

	@Override
	public String toString() {
		return String.format("%3d (%2.2f, %2.2f) r=%2.2f a=%2.2f c=%2.2f s=%2.2f", id, x, y, r, a, contrast, snr);
	}

	public String toStringComma() {
		String item[] = toArrayString();
		String s = "";
		for (int i = 0; i < item.length; i++)
			s += item[i] + ",";
		return s;
	}

	public String[] toArrayString() {
		String s = String.format("%05d", id);
		return new String[] { s, "" + x, "" + y, "" + r, "" + a, "" + contrast, "" + snr, type };
	}

	public static String[] toArrayStringHeader() {
		return new String[] { "No", "X", "Y", "R", "Confidence", "Contrast", "SNR", "Type" };
	}
}
