//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package spotcaliper.detector;

import ij.IJ;
import ij.ImagePlus;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

import scalablewavelets.Analysis;
import spotcaliper.gui.Data;
import spotcaliper.gui.components.ProgressionBar;
import spotcaliper.image2d.Image2DDouble;

public class SpotDetector {

	private Analysis		analysis;
	private Image2DDouble	workingImage;
	private ImagePlus		imp;
	private ProgressionBar	progress;
	private boolean			stop	= false;

	public SpotDetector(ImagePlus imp, ProgressionBar progress) {
		this.imp = imp;
		this.progress = progress;
	}

	public String getName() {
		return "Wavelet-based scale estimation";
	}

	public double[][] analysis(Parameters params) {
		workingImage = setWorkingImage(params.radiusMin, params.getBrightSpots());
		int nx = imp.getWidth();
		int ny = imp.getHeight();
		analysis = new Analysis(progress, nx, ny, params.radiusMax, params.delta, params.showAnalysis, params.multithread);
		analysis.analyze(workingImage.getData());
		double[][] map = analysis.computeMap();
		if (params.showAnalysis) {
			Image2DDouble imap = new Image2DDouble(map);
			new ImagePlus("map", imap.buildImageStack()).show();
		}
		return map;
	}

	public void detectAndSteer(double[][] map, Data data, Parameters params) {
		ArrayList<Spot> spotsSteered = new ArrayList<Spot>();
		double mapSmooth[][ ]= smoothGaussian(map, params.radiusMin*0.5);
		if (params.showAnalysis) {
			Image2DDouble imap = new Image2DDouble(mapSmooth);
			new ImagePlus("map smooth", imap.buildImageStack()).show();
		}
		
		ArrayList<double[]> localMaxPositions = findLocalMax3x3(mapSmooth, params.margin);
		if (stop)
			return;
		
		double f = Math.max(2, (params.radiusMax - params.radiusMin)*0.25);
		int n = (int)Math.min(f * params.nSpots, (map[0].length * map.length * 0.2));

		ArrayList<double[]> candidatesPositions = trim(localMaxPositions, n, params.radiusMin);
		if (stop)
			return;

		data.setLocalMax(candidatesPositions);

		int nc = candidatesPositions.size();
		int step = Math.max(1, nc / 100);
		for (int c = 0; c < nc; c++) {
			if (c % step == 0 && nc > 0)
				progress.progress("Steering " + c, c * 100. / nc);
			Spot spot = analysis.getOptimalScale(spotsSteered.size(), candidatesPositions.get(c));
			if (spot != null) {
				spot.id = spotsSteered.size();
				spotsSteered.add(spot);
			}
			if (stop)
				return;
		}

		ArrayList<Spot> spots = trim(spotsSteered, params.radiusMax, params.radiusMin, params.nSpots, -params.overlap);

		int count = 1;
		for (Spot spot : spots) {
			spot.measure(workingImage);
			spot.id = count++;
		}
		
		data.setSpots(spots, new ArrayList<Spot>());
		data.selectInitContrast(params.contrastMinimum);
	}

	private ArrayList<double[]> findLocalMax3x3(double[][] map, double margin) {
		int nx = map.length;
		int ny = map[0].length;
		int m = (int)Math.max(1, margin);
		ArrayList<double[]> locMaxPosition = new ArrayList<double[]>();
		for (int i = m; i < nx - m; ++i) {
			for (int j = m; j < ny - m; ++j) {
				double test = map[i][j];
				if (test > 0)
					if (map[i - 1][j - 1] <= test)
						if (map[i - 1][j] <= test)
							if (map[i - 1][j + 1] <= test)
								if (map[i + 1][j - 1] <= test)
									if (map[i + 1][j] <= test)
										if (map[i + 1][j + 1] <= test)
											if (map[i][j - 1] <= test)
												if (map[i][j + 1] <= test) {
													locMaxPosition.add(new double[] { i, j, test });
												}
			}
		}
		return locMaxPosition;
	}

	public ArrayList<double[]> trim(ArrayList<double[]> positions, int maxNumber, double proximity) {
		Collections.sort(positions, new Comparator<double[]>() {
			@Override
			public int compare(double[] position1, double[] position2) {
				return (position1[2] < position2[2] ? 1 : -1);

			}
		});

		ArrayList<double[]> goods = new ArrayList<double[]>();
		for (double[] position : positions) {
			boolean flag = false;
			for (double[] good : goods)
				if (distance(position, good) < proximity)
					flag = true;
			if (flag == false)
				goods.add(position);
			if (goods.size() >= maxNumber)
				return goods;
		}
		return goods;
	}

	public ArrayList<Spot> trim(ArrayList<Spot> spots, double rmax, double rmin, int maxNumber, double proximity) {
		Collections.sort(spots, new Comparator<Spot>() {
			@Override
			public int compare(Spot spot1, Spot spot2) {
				return (spot1.a < spot2.a ? 1 : -1);

			}
		});

		ArrayList<Spot> goods = new ArrayList<Spot>();
		for (Spot candidate : spots) {
			boolean flag = false;
			for (Spot good : goods)
				if (good.overlap(candidate) < proximity)
					flag = true;
			if ((flag == false) && (candidate.r < rmax * 1.2) && (candidate.r > rmin * 0.8))
				goods.add(candidate);
			if (goods.size() >= maxNumber)
				return goods;
		}
		return goods;
	}

	private double distance(double[] a, double[] b) {
		return Math.sqrt((a[0] - b[0]) * (a[0] - b[0]) + (a[1] - b[1]) * (a[1] - b[1]));
	}

	public void stop() {
		stop = true;
		analysis.stop();
	}

	public Spot getOptimalScale(int id, int x, int y) {
		Spot spot = analysis.getOptimalScale(id, new double[] { x, y, 0.0 });
		if (spot != null)
			spot.measure(workingImage);
		return spot;
	}

	private Image2DDouble setWorkingImage(double rmin, boolean brightSpots) {
		ImagePlus source = imp.duplicate();
		if (!brightSpots)
			source.getProcessor().invert();

		int nx = imp.getWidth();
		int ny = imp.getHeight();
		Image2DDouble image = new Image2DDouble(nx, ny);
		double data[] = image.getData();

		if (source.getType() == ImagePlus.GRAY8) {
			byte[] pix = (byte[]) (source.getProcessor().getPixels());
			for (int k = 0; k < pix.length; k++)
				data[k] = (double) (pix[k] & 0x00FF);
		}
		else if (source.getType() == ImagePlus.GRAY16) {
			short[] pix = (short[]) (source.getProcessor().getPixels());
			for (int k = 0; k < pix.length; k++)
				data[k] = (double) pix[k];
		}
		else {
			float[] pix = (float[]) (source.getProcessor().getPixels());
			for (int k = 0; k < pix.length; k++)
				data[k] = (double) pix[k];
		}
		return image;
	}

	public Image2DDouble getWorkingImage() {
		return workingImage;
	}
	
	private double[][] smoothGaussian(double map[][], double sigma) {
		int nx = map.length;
		int ny = map[0].length;
		double[][] out = new double[nx][ny];
		int n = 3;
		double N = (double) n;
		double poles[] = new double[n];
		double s2 = sigma;
		double alpha = 1.0 + (N / s2) - (Math.sqrt(N * N + 2 * N * s2) / s2);
		poles[0] = poles[1] = poles[2] = alpha;
		
	
		double line[] = new double[nx];
		for (int y = 0; y < ny; y++) {
			for (int x = 0; x < nx; x++)
				line[x] = map[x][y];
			line = Convolver.convolveIIR(line, poles);
			for (int x = 0; x < nx; x++) 
				out[x][y] = line[x];
		}
		
		for (int x = 0; x < nx; x++)
			out[x] = Convolver.convolveIIR(out[x], poles);
		return out;
	
	}


}