//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package spotcaliper.gui;

import ij.IJ;
import ij.ImagePlus;
import ij.gui.ImageCanvas;
import ij.gui.ImageWindow;
import ij.gui.StackWindow;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

import spotcaliper.detector.RunningMode;
import spotcaliper.detector.Spot;
import spotcaliper.detector.SpotDetector;
import spotcaliper.gui.components.ColorName;
import spotcaliper.gui.components.InfoPane;
import spotcaliper.image2d.Image2DDouble;

public class CanvasSelection extends ImageCanvas implements Runnable, KeyListener {

	private ImageCanvas		canvasOriginal;
	private SpotDetector	detector;
	private Data			data;
	private boolean[]		shows				= new boolean[] { true, false, false, false };
	private Color[]			colors				= new Color[] { ColorName.getColor("Red"), ColorName.getColor("Lime"), ColorName.getColor("Red"), ColorName.getColor("Red") };
	private int				stroke				= 2;
	private int				opacity				= 50;
	private String			label				= "None";
	private Image2DDouble	input;
	private int				selectedID			= -1;

	// For the double buffering
	private Dimension		dim;
	private Image			offscreen;
	private Graphics2D		bufferGraphics;

	private int				xmouse, ymouse;
	private Thread			thread				= null;
	private DialogSelection	dialog;

	private Cursor			cursor;
	private Cursor			hand;

	private boolean			isAlreadyOneClick	= false;
	private InfoPane		info;
	private int				optimalCenter;
	private int				optimalRadius;
	private boolean			automatic			= true;
	private boolean			manual				= true;
	private boolean			specificColorManual = false;
	
	public CanvasSelection(ImagePlus imp, SpotDetector detector, Data data, DialogSelection dialog, InfoPane info) {
		super(imp);
		input = new Image2DDouble(imp);
		this.detector = detector;
		this.data = data;
		this.dialog = dialog;
	
		if (imp.getStackSize() > 1)
			imp.setWindow(new StackWindow(imp, this));
		else
			imp.setWindow(new ImageWindow(imp, this));
		resetBuffer();
		cursor = this.getCursor();
		hand = new Cursor(Cursor.HAND_CURSOR);

		ImageWindow window = imp.getWindow();
		canvasOriginal = imp.getCanvas();
		window.removeKeyListener(IJ.getInstance());
		canvasOriginal.removeKeyListener(IJ.getInstance());
		canvasOriginal.addKeyListener(this);
		window.addKeyListener(this);
		this.info = info;
	}

	public void setOptimize(int optimalCenter, int optimalRadius) {
		this.optimalCenter = optimalCenter;
		this.optimalRadius = optimalRadius;
	}

	public void setFlag(boolean automatic, boolean manual) {
		this.automatic = automatic;
		this.manual = manual;
		repaint();
	}

	public void setSelected(int selectedID) {
		this.selectedID = selectedID;
		repaint();
	}
	
	public void forceSpecificColorManual() {
		specificColorManual = true;
	}
	
	@Override
	public void paint(Graphics g) {

		if (shows[0] == false && shows[1] == false && shows[2] == false && shows[3] == false) {
			super.paint(g);
			return;
		}

		if (dim.width != getSize().width || dim.height != getSize().height || bufferGraphics == null || offscreen == null)
			resetBuffer();

		super.paint(bufferGraphics);
	
		double mag = this.getMagnification();
		int o = (int) (2.55 * opacity);

		Color inner = ColorName.opacify(colors[1], o);
		Color outer = colors[0];
		ArrayList<Spot> spots = data.getSelectedAndManual(automatic, manual);

		for (int i = 0; i < spots.size(); i++) {
			Spot spot = spots.get(i);
			if (specificColorManual)
				if (spot.isManual()) 
					outer = Color.GREEN;
			if (spot.id != selectedID)
				drawSpot(bufferGraphics, spot, outer, inner, mag);
			else
				drawSpot(bufferGraphics, spot, ColorName.inverse(outer), ColorName.inverse(inner, o), mag);
		}
		
		if (specificColorManual)
			for(double[] loc : data.getLocalMax())
				drawCross(bufferGraphics, loc, mag);
		
		g.drawImage(offscreen, 0, 0, this);
	}

	private void remove() {
		selectedID = data.findSpotID(xmouse, ymouse, automatic, manual);
		if (selectedID >= 0) {
			data.remove(selectedID);
			if (dialog != null)
				dialog.update(-1);
			if (info != null)
				info.append("Remove spot " + selectedID);
		}
		repaint();
	}

	private void add(int xmouse, int ymouse) {
		Spot spot = detector.getOptimalScale(Integer.MAX_VALUE, xmouse, ymouse);
		if (spot != null) {
			spot.measure(input);
			data.add(spot);
			if (dialog != null)
				dialog.update(spot.id);
		}
		repaint();
		if (info != null)
			info.append("Add spot " + selectedID + " at (" + xmouse + ", " + ymouse + ")");
	}

	public void setDisplay(boolean shows[], Color colors[], int stroke, int opacity, String label) {
		this.shows = shows;
		this.colors = colors;
		this.stroke = stroke;
		this.opacity = opacity;
		this.label = label;
		repaint();
	}

	public void resetBuffer() {
		if (bufferGraphics != null) {
			bufferGraphics.dispose();
			bufferGraphics = null;
		}
		if (offscreen != null) {
			offscreen.flush();
			offscreen = null;
		}
		dim = getSize();
		offscreen = createImage(dim.width, dim.height);
		bufferGraphics = (Graphics2D) offscreen.getGraphics();
	}

	public void drawSpot(Graphics2D g2, Spot spot, Color outer, Color inner, double mag) {
		int rm = (int) Math.round(mag * spot.r);
		if (shows[1]) {
			g2.setColor(inner);
			g2.fillOval(screenXD(spot.x) - rm, screenYD(spot.y) - rm, 2 * rm, 2 * rm);
		}
		if (shows[0]) {
			g2.setColor(outer);
			g2.setStroke(new BasicStroke(stroke));
			g2.drawOval(screenXD(spot.x) - rm, screenYD(spot.y) - rm, 2 * rm, 2 * rm);
		}
		if (shows[2]) {
			if (label.equals("ID"))
				print(g2, screenXD(spot.x), screenYD(spot.y), "" + spot.id, colors[2]);
			else if (label.equals("Confidence"))
				print(g2, screenXD(spot.x), screenYD(spot.y), String.format("%2.2f", spot.a), colors[2]);
			else if (label.equals("Radius"))
				print(g2, screenXD(spot.x), screenYD(spot.y), String.format("%2.2f", spot.r), colors[2]);
			else if (label.equals("Contrast"))
				print(g2, screenXD(spot.x), screenYD(spot.y), String.format("%2.2f", spot.contrast), colors[2]);
			else if (label.equals("SNR"))
				print(g2, screenXD(spot.x), screenYD(spot.y), String.format("%2.2f", spot.snr), colors[2]);
		}
		if (shows[3]) {
			g2.setColor(colors[3]);
			g2.setStroke(new BasicStroke(1));
			g2.drawLine(screenXD(spot.x) - rm, screenYD(spot.y), screenXD(spot.x) + rm, screenYD(spot.y));
			g2.drawLine(screenXD(spot.x), screenYD(spot.y) - rm, screenXD(spot.x), screenYD(spot.y) + rm);
		}
	}

	public void drawCross(Graphics2D g2, double[] loc, double mag) {
		int rm = (int) Math.round(mag * 5);
		g2.setColor(colors[3]);
		g2.setStroke(new BasicStroke(1));
		g2.drawLine(screenXD(loc[0]) - rm, screenYD(loc[1]), screenXD(loc[0]) + rm, screenYD(loc[1]));
		g2.drawLine(screenXD(loc[0]), screenYD(loc[1]) - rm, screenXD(loc[0]), screenYD(loc[1]) + rm);
		
	}

	public void print(Graphics g2, int x, int y, String message, Color fore) {
		double distWhite = Math.abs(255 - fore.getGreen()) + Math.abs(255 - fore.getRed()) + Math.abs(255 - fore.getBlue());
		double distBlack = fore.getGreen() + fore.getRed() + fore.getBlue();
		g2.setColor(distWhite < distBlack ? Color.WHITE : Color.BLACK);
		g2.drawString(message, x + 1, y + 1);
		g2.setColor(fore);
		g2.drawString(message, x, y);
	}

	@Override
	public void run() {
		//cmbBestCenter	= new JComboBox(new String[] { "Fixed position", "Best center in 3x3", "Best center in 5x5", "7x7", "9x9", "Best center in 11x11", "Best center in 13x13" });
		//cmbBestRadius	= new JComboBox(new String[] { "Optimal radius", "Fixed radius" });
		int n = optimalCenter; 
		double max = -Double.MAX_VALUE;
		int imax = xmouse;
		int jmax = ymouse;
		if (n>0)
		for (int i = xmouse - n; i <= xmouse + n; i++)
			for (int j = ymouse - n; j <= ymouse + n; j++) {
				Spot spotTest = detector.getOptimalScale(selectedID, i, j);
				if (spotTest != null)
					if (spotTest.a > max) {
						max = spotTest.a;
						imax = i;
						jmax = j;
					}
			}
		Spot spot = data.findSpot(selectedID);
		if (spot != null) {	
			if (optimalRadius == 0) {
				Spot spotNew = detector.getOptimalScale(selectedID, imax, jmax);
				if (spotNew != null)
					data.move(selectedID, spotNew.x, spotNew.y, spotNew.r, spotNew.a, input);
			}
			else {
				data.move(selectedID, imax, jmax, spot.r, spot.a, input);
			}
			if (dialog != null)
				dialog.updateTable(selectedID);
			if (dialog != null)
				dialog.updateSpot(spot);
		}	
		if (dialog != null)
			dialog.updateCount();
		repaint();
		thread = null;
	}

	@Override
	public void mouseMoved(MouseEvent event) {
		setCursor(selectedID >= 0 ? hand : cursor);
		super.mouseMoved(event);
	}

	@Override
	public void mouseReleased(MouseEvent event) {
		selectedID = -1;
		super.mouseReleased(event);
	}

	@Override
	public void mouseDragged(MouseEvent event) {
		if (selectedID >= 0) {
			xmouse = offScreenX(event.getX());
			ymouse = offScreenY(event.getY());
			if (thread == null) {
				thread = new Thread(this);
				thread.setPriority(Thread.MIN_PRIORITY);
				thread.start();
			}
		}
		else {
			super.mouseDragged(event);
		}
	}

	@Override
	public void mousePressed(final MouseEvent event) {
		if (isAlreadyOneClick) {
			mousePressedDoubleClick(event);
			isAlreadyOneClick = false;
			return;
		}
		// Toolkit.getDefaultToolkit().getDesktopProperty("awt.multiClickInterval").intValue();
		int time = 200; // ms
		isAlreadyOneClick = true;
		Timer t = new Timer("doubleclickTimer", false);
		t.schedule(new TimerTask() {
			@Override
			public void run() {
				if (isAlreadyOneClick)
					mousePressedSimpleClick(event);
				isAlreadyOneClick = false;
			}
		}, time);
	}

	private void mousePressedSimpleClick(MouseEvent event) {
		xmouse = offScreenX(event.getX());
		ymouse = offScreenY(event.getY());
		selectedID = data.findSpotID(xmouse, ymouse, automatic, manual);
		if (selectedID >= 0) {
			dialog.updateCanvas(selectedID);
			return;
		}
		super.mousePressed(event);
	}

	private void mousePressedDoubleClick(MouseEvent event) {
		xmouse = offScreenX(event.getX());
		ymouse = offScreenY(event.getY());
		selectedID = data.findSpotID(xmouse, ymouse, automatic, manual);
		if (selectedID < 0)
			add(xmouse, ymouse);
	}

	@Override
	public void keyPressed(KeyEvent event) {
		if (event.getKeyCode() == KeyEvent.VK_BACK_SPACE || event.getKeyCode() == KeyEvent.VK_DELETE)
			remove();
	}

	@Override
	public void keyReleased(KeyEvent event) {
	}

	@Override
	public void keyTyped(KeyEvent event) {
	}

}
