//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package spotcaliper.gui;

import ij.ImagePlus;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.StringTokenizer;

import spotcaliper.detector.Spot;
import spotcaliper.gui.components.SpinnerDouble;
import spotcaliper.image2d.Image2DDouble;

public class Data {

	private ArrayList<Spot>		spotsDetected			= new ArrayList<Spot>();
	private ArrayList<Spot>		spotsSelected 			= new ArrayList<Spot>();
	private ArrayList<Spot>		spotsManual				= new ArrayList<Spot>();
	private ArrayList<double[]>	localMax				= new ArrayList<double[]>();

	private double				minValueFeature[]		= new double[6];
	private double				maxValueFeature[]		= new double[6];
	private String				mostSignificantFeature	= "";
	private int					mostSignificantNumber	= 999999;

	private ImagePlus			imp;

	private double				lowerFeature[]			= new double[6];
	private double				upperFeature[]			= new double[6];

	public Data(ImagePlus imp) {
		this.imp = imp;
	}

	public void setLocalMax(ArrayList<double[]> localMax) {
		this.localMax = localMax;
	}

	public ArrayList<double[]> getLocalMax() {
		return localMax;
	}

	public ArrayList<Spot> getSelected() {
		return spotsSelected;
	}

	public ArrayList<Spot> getDetected() {
		return spotsDetected;
	}

	public ArrayList<Spot> getManual() {
		return spotsManual;
	}

	public ArrayList<Spot> getSelectedAndManual(boolean automatic, boolean manual) {
		ArrayList<Spot> spotsAll = new ArrayList<Spot>();
		if (automatic)
			for (Spot spot : spotsSelected)
				spotsAll.add(spot);
		if (manual)
			for (Spot spot : spotsManual)
				spotsAll.add(spot);
		return spotsAll;
	}

	public void setSpots(ArrayList<Spot> spotsAutomatic, ArrayList<Spot> spotsManual_) {
		spotsSelected = new ArrayList<Spot>();
		spotsDetected = new ArrayList<Spot>();
		for (Spot spot : spotsAutomatic) {
			spotsDetected.add(spot);
			spotsSelected.add(spot);
		}
		spotsManual = new ArrayList<Spot>();
		for (Spot spot : spotsManual_) {
			spotsManual.add(spot);
		}

		ArrayList<Spot> spotsAll = getSelectedAndManual(true, true);

		for (int i = 0; i < lowerFeature.length; i++) {
			lowerFeature[i] = Double.MAX_VALUE;
			maxValueFeature[i] = -Double.MAX_VALUE;
		}

		for (Spot spot : spotsAll) {
			double[] feature = spot.getFeature();
			for (int i = 0; i < upperFeature.length; i++)
				lowerFeature[i] = Math.min(lowerFeature[i], feature[i]);
			for (int i = 0; i < upperFeature.length; i++)
				upperFeature[i] = Math.max(upperFeature[i], feature[i]);
		}

		lowerFeature[0] = 0;
		lowerFeature[1] = 0;
		maxValueFeature[0] = imp.getWidth() + 1;
		maxValueFeature[1] = imp.getHeight() + 1;

		resetRangeValue();
	}

	public Spot findSpotInDetect(int id) {
		for (Spot spot : spotsDetected)
			if (spot.id == id)
				return spot;
		return null;
	}

	public Spot findSpot(int id) {
		for (Spot spot : spotsDetected)
			if (spot.id == id)
				return spot;
		for (Spot spot : spotsManual)
			if (spot.id == id)
				return spot;
		for (Spot spot : spotsSelected)
			if (spot.id == id)
				return spot;
		return null;
	}

	public int findSpotID(int xm, int ym, boolean automatic, boolean manual) {
		double dAuto = Double.MAX_VALUE;
		int idAuto = -1;
		if (automatic)
			for (Spot spot : spotsSelected)
				if (spot.contains(xm, ym)) {
					double d = spot.distance(xm, ym);
					if (d < dAuto) {
						idAuto = spot.id;
						dAuto = d;
					}
				}

		double dManual = Double.MAX_VALUE;
		int idManual = -1;
		if (manual)
			for (Spot spot : spotsManual)
				if (spot.contains(xm, ym)) {
					double d = spot.distance(xm, ym);
					if (d < dManual) {
						idManual = spot.id;
						dManual = d;
					}
				}

		if (idAuto == -1) {
			if (idManual == -1)
				return -1;
			else
				return idManual;
		}
		else {
			if (idManual == -1)
				return idAuto;
			else
				return (dAuto < dManual ? idAuto : idManual);
		}
	}

	public void move(int id, double x, double y, double r, double a, Image2DDouble input) {
		
		for (Spot spot : spotsDetected)
			if (spot.id == id) {
				spot.x = x;
				spot.y = y;
				spot.r = r;
				spot.a = a;
				spot.setManual();
				spot.measure(input);
				spotsDetected.remove(spot);
				spotsSelected.remove(spot);
				spotsManual.add(spot);
				return;
			}
			
		for (Spot spot : spotsManual)
			if (spot.id == id) {
				spot.x = x;
				spot.y = y;
				spot.r = r;
				spot.a = a;
				spot.measure(input);
				return;
			}
	}

	public void remove(int id) {
		remove(id, spotsSelected);
		remove(id, spotsManual);
	}

	public void remove(int id, ArrayList<Spot> spots) {
		Spot delete = null;
		for (Spot spot : spots)
			if (spot.id == id)
				delete = spot;
		if (delete != null)
			spots.remove(delete);
	}

	public void add(Spot spot) {
		spot.setManual();
		int max = 0;
		for (Spot s : spotsManual)
			if (s.id > max)
				max = s.id;
		for (Spot s : spotsDetected)
			if (s.id > max)
				max = s.id;
		for (Spot s : spotsSelected)
			if (s.id > max)
				max = s.id;
		
		spot.id = max+1;
		spotsManual.add(spot);
	}

	public SpinnerDouble getLowerSpinner(int i) {
		double a = lowerFeature[i];
		double b = upperFeature[i];
		double step = 0;
		if (b > a) {
			step = (b - a) / 100.0;
		}
		return new SpinnerDouble(minValueFeature[i], a - step, b + step, step);
	}

	public SpinnerDouble getUpperSpinner(int i) {
		double a = lowerFeature[i];
		double b = upperFeature[i];
		double step = 0;
		if (b > a) {
			step = (b - a) / 100.0;
		}
		return new SpinnerDouble(maxValueFeature[i], a - step, b + step, step);
	}

	public void setLimitValue(int i, double min, double max) {
		maxValueFeature[i] = max;
		minValueFeature[i] = min;
	}

	public boolean loadCVS(String filename) {
		String line = "";
		try {
			BufferedReader buffer = new BufferedReader(new FileReader(filename));
			line = buffer.readLine();
			line = buffer.readLine();
			ArrayList<Spot> spotsAutoLoaded = new ArrayList<Spot>();
			ArrayList<Spot> spotsManualLoaded = new ArrayList<Spot>();
			while (line != null) {
				StringTokenizer tokens = new StringTokenizer(line, ",");
				int id = Integer.parseInt(tokens.nextToken().trim());
				int x = (int) Double.parseDouble(tokens.nextToken().trim());
				int y = (int) Double.parseDouble(tokens.nextToken().trim());
				double r = Double.parseDouble(tokens.nextToken().trim());
				double a = Double.parseDouble(tokens.nextToken().trim());
				double contrast = Double.parseDouble(tokens.nextToken().trim());
				double snr = Double.parseDouble(tokens.nextToken().trim());
				String type = tokens.nextToken().trim();
				if (type.startsWith("A"))
					spotsAutoLoaded.add(new Spot(id, x, y, r, a, contrast, snr, type));
				else
					spotsManualLoaded.add(new Spot(id, x, y, r, a, contrast, snr, type));
				line = buffer.readLine();
			}
			buffer.close();
			setSpots(spotsAutoLoaded, spotsManualLoaded);
			return true;
		}
		catch (Exception ex) {
			System.out.println("Unable to read the file " + line);
		}
		return false;
	}

	public void saveCVS(String filename) {
		File file = new File(filename);
		try {
			String headers[] = Spot.toArrayStringHeader();
			BufferedWriter buffer = new BufferedWriter(new FileWriter(file));
			String s = "";
			for (int i = 0; i < headers.length; i++)
				s += headers[i] + ",";
			buffer.write(s + "\n");
			for (Spot spot : spotsSelected)
				buffer.write(spot.toStringComma() + "\n");
			for (Spot spot : spotsManual)
				buffer.write(spot.toStringComma() + "\n");
			buffer.close();
		}
		catch (IOException ex) {
			System.out.println("" + ex);
		}
	}

	public String getSignificantFeature() {
		return mostSignificantFeature;
	}

	public int getSignificantNumber() {
		return mostSignificantNumber;
	}

	public void setSignificantFeature(String feature) {
		mostSignificantFeature = feature;
	}

	public void setSignificantNumber(int number) {
		mostSignificantNumber = number;
	}

	public void copyRangeValuesTo(double[] min, double max[]) {
		for (int i = 0; i < min.length; i++) {
			min[i] = minValueFeature[i];
			max[i] = maxValueFeature[i];
		}
	}

	public void copyRangeValuesFrom(double[] min, double max[]) {
		for (int i = 0; i < min.length; i++) {
			minValueFeature[i] = min[i];
			maxValueFeature[i] = max[i];
		}
	}

	public void resetRangeValue() {
		for (int i = 0; i < maxValueFeature.length; i++) {
			minValueFeature[i] = lowerFeature[i];
			maxValueFeature[i] = upperFeature[i];
		}
	}

	public void changeRatioSelected(boolean increase) {
		int m = spotsSelected.size();
		int n = spotsDetected.size();
		if (n > 0) {
			double inc = (increase ? Math.max(1, (n-m)*0.1) : Math.min(-1, (0-m)*0.1));
			select("Contrast", (int)Math.round(m + inc));
		}
	}

	public void select(String mostSignificantFeature, int mostSignificantNumber) {
		this.mostSignificantFeature = mostSignificantFeature;
		this.mostSignificantNumber = mostSignificantNumber;
		select();
	}

	public void selectInitContrast(double contrastMinimum) {
		minValueFeature[4] = contrastMinimum;
		select();
		resetRangeValue();
	}

	public void select(double minValueFeature[], double maxValueFeature[]) {
		this.minValueFeature = minValueFeature;
		this.maxValueFeature = maxValueFeature;
		select();

	}

	public void selectAll() {
		resetRangeValue();
		mostSignificantFeature = "";
	}

	private void select() {

		spotsSelected.clear();
		String feature = mostSignificantFeature;

		if (feature.equals("X"))
			sortX();
		if (feature.equals("Y"))
			sortY();
		if (feature.equals("Radius"))
			sortRadius();
		if (feature.equals("Confidence"))
			sortConfidence();
		if (feature.equals("Contrast"))
			sortContrast();
		if (feature.equals("SNR"))
			sortSNR();

		ArrayList<Spot> spotsSignificant = new ArrayList<Spot>();
		for (int i = 0; i < Math.min(mostSignificantNumber, spotsDetected.size()); i++) {
			spotsSignificant.add(spotsDetected.get(i));
		}

		for (Spot spot : spotsSignificant) {
			if (!spot.isManual()) {
				if (spot.x >= minValueFeature[0] && spot.x <= maxValueFeature[0])
					if (spot.y >= minValueFeature[1] && spot.y <= maxValueFeature[1])
						if (spot.r >= minValueFeature[2] && spot.r <= maxValueFeature[2])
							if (spot.a >= minValueFeature[3] && spot.a <= maxValueFeature[3])
								if (spot.contrast >= minValueFeature[4] && spot.contrast <= maxValueFeature[4])
									if (spot.snr >= minValueFeature[5] && spot.snr <= maxValueFeature[5])
										spotsSelected.add(spot);
			}
		}

	}

	private void sortX() {
		Collections.sort(spotsDetected, new Comparator<Spot>() {
			@Override
			public int compare(Spot spot1, Spot spot2) {
				return (spot1.x < spot2.x ? 1 : -1);
			}
		});
	}

	public void sortY() {
		Collections.sort(spotsDetected, new Comparator<Spot>() {
			@Override
			public int compare(Spot spot1, Spot spot2) {
				return (spot1.y < spot2.y ? 1 : -1);
			}
		});
	}

	private void sortRadius() {
		Collections.sort(spotsDetected, new Comparator<Spot>() {
			@Override
			public int compare(Spot spot1, Spot spot2) {
				return (spot1.r < spot2.r ? 1 : -1);
			}
		});

	}

	private void sortConfidence() {
		Collections.sort(spotsDetected, new Comparator<Spot>() {
			@Override
			public int compare(Spot spot1, Spot spot2) {
				return (spot1.a < spot2.a ? 1 : -1);
			}
		});

	}

	private void sortContrast() {
		Collections.sort(spotsDetected, new Comparator<Spot>() {
			@Override
			public int compare(Spot spot1, Spot spot2) {
				return (spot1.contrast < spot2.contrast ? 1 : -1);
			}
		});

	}

	private void sortSNR() {
		Collections.sort(spotsDetected, new Comparator<Spot>() {
			@Override
			public int compare(Spot spot1, Spot spot2) {
				return (spot1.snr < spot2.snr ? 1 : -1);
			}
		});

	}

}
