//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package spotcaliper.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import spotcaliper.detector.Spot;

public class SpotTable extends JTable {

	private Color			colorOddRow		= new Color(245, 245, 250);
	private Color			colorEvenRow	= new Color(232, 232, 237);

	private String[]		tooltip			= new String[] { "" + "Identifier of the spot", "Horizontal coordinate of the spot (in pixels)", "Vertical coordinate of the spot (in pixels)",
			"Radius of the spot (in pixel)", "Confidence level: strength of the wavelet coefficient", "Contrast: measures of the distinction between the spot and its background",
			"SNR: signal-to-noise ratio", "Type of insertion" };
	private String[]		headers			= new String[] { "ID", "X", "Y", "Radius", "Confidence", "Contrast", "SNR", "Type" };

	public SpotTable(Data data) {
		super();
		DefaultTableModel
		tableModel = new DefaultTableModel() {
			@Override
			public boolean isCellEditable(int row, int column) {
				return false;
			}

			@Override
			public Class<?> getColumnClass(int column) {
				if (column <= 2)
					return Integer.class;
				else if (column <= 6)
					return Double.class;
				return super.getColumnClass(column);
			}
		};

		setModel(tableModel);
		DefaultTableModel model = (DefaultTableModel) getModel();
		model.setColumnIdentifiers(headers);
		setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		setRowSelectionAllowed(true);
		setAutoCreateRowSorter(true);

		for (int i = 0; i < headers.length; i++) {
			TableColumn tc = getColumnModel().getColumn(i);
			tc.setCellRenderer(new AlternatedRowRenderer());
		}
		update(data.getSelectedAndManual(true, true));

		JTableHeader header = getTableHeader();
		ColumnHeaderToolTips tips = new ColumnHeaderToolTips();
		for (int c = 0; c < getColumnCount(); c++) {
			TableColumn col = getColumnModel().getColumn(c);
			tips.setToolTip(col, tooltip[c]);
		}
		header.addMouseMotionListener(tips);

/*
		getModel().addTableModelListener(new TableModelListener() {
			@Override
			public void tableChanged(TableModelEvent e) {
				nRows = getRowCount();
IJ.log("event2 " + selectedRow + " " + nRows);						

				SwingUtilities.invokeLater(new Runnable() {
					@Override
					public void run() {
						if (selectedRow >= 0 && selectedRow < nRows)
							addRowSelectionInterval(selectedRow, selectedRow);
					}
				});
			}
		});*/

	}

	@Override
	public Component prepareRenderer(TableCellRenderer renderer, int row, int column) {
		Component component = super.prepareRenderer(renderer, row, column);
		int rendererWidth = component.getPreferredSize().width;
		TableColumn tableColumn = getColumnModel().getColumn(column);
		if (column <= 2)
			tableColumn.setPreferredWidth(34);
		else if (column <= 5)
			tableColumn.setPreferredWidth(59);
		else
			tableColumn.setPreferredWidth(Math.max(rendererWidth + 1, tableColumn.getPreferredWidth()));
		return component;
	}

	public JScrollPane getPane(int width, int height) {
		JScrollPane scrollpane = new JScrollPane(this);
		scrollpane.setPreferredSize(new Dimension(width, height));
		return scrollpane;
	}

	public int getID(int row) {
		if (row < 0)
			return -1;
		if (row >= getRowCount())
			return -1;
		return (Integer)getValueAt(row, 0);
	}

	public int getRow(int id) {
		for (int row = 0; row < getRowCount(); row++) {
			if (((Integer)getValueAt(row, 0)) == id)
				return row;
		}
		return -1;
	}

	public void update(ArrayList<Spot> spots) {
		DefaultTableModel model = (DefaultTableModel) getModel();
		model.getDataVector().removeAllElements();
		for (Spot spot : spots)
			addSpot(spot);
		repaint();
	}

	public void update(Spot spot) {
		DefaultTableModel model = (DefaultTableModel) getModel();
		int row = getRow(spot.id);	
		if (row < 0)
			return;

		model.setValueAt(spot.x, row, 1);
		model.setValueAt(spot.y, row, 2);
		model.setValueAt(round(spot.r, 4), row, 3);
		model.setValueAt(round(spot.a, 4), row, 4);
		model.setValueAt(round(spot.contrast, 4), row, 5);
		model.setValueAt(round(spot.snr, 4), row, 6);
		model.setValueAt(spot.getType(), row, 7);
		repaint();
	}

	private double round(double num, int n) {
		if (num == 0.0)
			return 0.0;
		double d = Math.ceil(Math.log10(num < 0.0 ? -num : num));
		int power = n - (int) d;
		double a = Math.pow(10, power);
		long shifted = Math.round(num * a);
		return shifted / a;
	}

	private void addSpot(Spot spot) {
		Object[] tmp = new Object[] { spot.id, spot.x, spot.y, round(spot.r, 4), round(spot.a, 4), round(spot.contrast, 4), round(spot.snr, 5), spot.getType() };
		DefaultTableModel model = (DefaultTableModel) getModel();
		model.addRow(tmp);
	}

	public class AlternatedRowRenderer extends DefaultTableCellRenderer {
		@Override
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int col) {
			super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);
			Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);
			if (!isSelected)
				c.setBackground(row % 2 == 0 ? colorEvenRow : colorOddRow);
			if (c instanceof JComponent)
				((JComponent) c).setToolTipText(tooltip[col]);
			return c;
		}
	}

	class ColumnHeaderToolTips extends MouseMotionAdapter {
		TableColumn						curCol;
		HashMap<TableColumn, String>	tips	= new HashMap<TableColumn, String>();

		public void setToolTip(TableColumn col, String tooltip) {
			if (tooltip == null)
				tips.remove(col);
			else
				tips.put(col, tooltip);
		}

		@Override
		public void mouseMoved(MouseEvent evt) {
			JTableHeader header = (JTableHeader) evt.getSource();
			JTable table = header.getTable();
			TableColumnModel colModel = table.getColumnModel();
			int vColIndex = colModel.getColumnIndexAtX(evt.getX());
			TableColumn col = null;
			if (vColIndex >= 0) {
				col = colModel.getColumn(vColIndex);
			}
			if (col != curCol) {
				header.setToolTipText((String) tips.get(col));
				curCol = col;
			}
		}
	}

}
